from gradio import themes

# Generated pallette using https://www.tints.dev/blue/2075BC
THEME_HUE = themes.Color(
    "#E5F1FB",
    "#CFE5F7",
    "#9BC9EE",
    "#6BAEE6",
    "#3792DD",
    "#2075BC",
    "#195D94",
    "#134771",
    "#0D2E4A",
    "#071927",
    "#030B11",
    "UHG",
)

LOGO = "https://upload.wikimedia.org/wikipedia/commons/thumb/f/f3/UnitedHealth_Group_logo.svg/444px-UnitedHealth_Group_logo.svg.png"
NAME = "UHG"
